"use strict";
var VariableResolverService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.VariableResolverService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
let VariableResolverService = VariableResolverService_1 = class VariableResolverService {
    /**
     * 处理给定的变量数组
     * @param value 需要处理的变量数组
     * @param options 处理参数
     */
    resolveArray(value, options = {}) {
        return this.resolve(value, options);
    }
    /**
     * 变量处理函数
     * @param value 变量
     * @param options 处理参数
     */
    async resolve(value, options = {}) {
        const context = new VariableResolverService_1.Context(this.variableRegistry, options);
        const resolved = await this.doResolve(value, context);
        return resolved;
    }
    /**
     * 实际处理函数
     * @param value
     * @param context
     */
    async doResolve(value, context) {
        if (value === undefined || value === null) {
            return value;
        }
        if (typeof value === 'string') {
            return this.doResolveString(value, context);
        }
        if (Array.isArray(value)) {
            return this.doResolveArray(value, context);
        }
        if (typeof value === 'object') {
            return this.doResolveObject(value, context);
        }
        return value;
    }
    /**
     * 获取对象值
     * @param obj
     * @param context
     */
    async doResolveObject(obj, context) {
        const result = {};
        for (const name of Object.keys(obj)) {
            const value = obj[name];
            const resolved = await this.doResolve(value, context);
            result[name] = resolved;
        }
        return result;
    }
    /**
     * 获取数组值
     * @param values
     * @param context
     */
    async doResolveArray(values, context) {
        const result = [];
        for (const value of values) {
            const resolved = await this.doResolve(value, context);
            result.push(resolved);
        }
        return result;
    }
    /**
     * 获取字符串值
     * @param value
     * @param context
     */
    async doResolveString(value, context) {
        await this.resolveVariables(value, context);
        return value.replace(VariableResolverService_1.VAR_REGEXP, (match, varName) => {
            const varValue = context.get(varName);
            return varValue !== undefined ? varValue : match;
        });
    }
    async resolveVariables(value, context) {
        let match;
        while ((match = VariableResolverService_1.VAR_REGEXP.exec(value)) !== null) {
            const variableName = match[1];
            await context.resolve(variableName);
        }
    }
};
VariableResolverService.VAR_REGEXP = /\$\{(.*?)\}/g;
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.VariableRegistry),
    tslib_1.__metadata("design:type", Object)
], VariableResolverService.prototype, "variableRegistry", void 0);
VariableResolverService = VariableResolverService_1 = tslib_1.__decorate([
    (0, di_1.Injectable)()
], VariableResolverService);
exports.VariableResolverService = VariableResolverService;
(function (VariableResolverService) {
    class Context {
        constructor(variableRegistry, options) {
            this.variableRegistry = variableRegistry;
            this.options = options;
            this.resolved = new Map();
        }
        async evaluateSingleVariable(name) {
            let variable;
            const parts = name.split(':');
            if (parts.length > 1) {
                const [key, value] = parts;
                variable = this.variableRegistry.getVariable(key);
                switch (key) {
                    case 'env': {
                        const environment = variable && (await variable.resolve(this.options.context));
                        if (!environment) {
                            return;
                        }
                        const env = environment[ide_core_browser_1.isWindows ? value.toLowerCase() : value];
                        return env;
                    }
                    default:
                        break;
                }
            }
            else {
                const variable = this.variableRegistry.getVariable(name);
                const value = variable && (await variable.resolve(this.options.context));
                return value;
            }
        }
        get(name) {
            return this.resolved.get(name);
        }
        async resolve(name) {
            if (this.resolved.has(name)) {
                return;
            }
            try {
                const value = await this.evaluateSingleVariable(name);
                this.resolved.set(name, value);
            }
            catch (e) {
                this.resolved.set(name, undefined);
            }
        }
    }
    VariableResolverService.Context = Context;
})(VariableResolverService = exports.VariableResolverService || (exports.VariableResolverService = {}));
exports.VariableResolverService = VariableResolverService;
//# sourceMappingURL=variable-resolver.service.js.map